/*********************************************************
* tabControl.js                                          *
*                                                        *
* Creates and activates tabcontrols in HTML              *
*                                                        *
* Tabcontrols must be included like this:                *
* <DIV class="TabCtrl">                                  *
* <DIV class="TabCtrlTab">tab 1 title</DIV>              *
* <DIV class="TabCtrlPanel">tab 1 contents</DIV>         *
* <DIV class="TabCtrlTab">tab 2 title</DIV>              *
* <DIV class="TabCtrlPanel">tab 2 contents</DIV>         *
* ...                                                    *
* <DIV class="TabCtrlTab">tab N title</DIV>              *
* <DIV class="TabCtrlPanel">tab N contents</DIV>         *
* </DIV>                                                 *
*                                                        *
* The number of controls on a page is unlimited.         *
* Controls may be embedded in others (e.g. tables).      *
*                                                        *
* This code replaces the placeholder above by a table.   *
* The elements within this table are given the classes:  *
* - TabCtrl (the table), Tab, ActiveTab, Panel, and      *
*   ActivePanel (the tab content text)                   *
* Formatting of these elements must be done entirely     *
* using a stylesheet.                                    *
*********************************************************/

/* Call this function once at 'onload' for the body of the page.
 * It scans the page for tab controls and replaces the DIV elements
 * by tables with the desired behaviour.
 */
function createTabs()
{
    var divs = document.getElementsByTagName('DIV');

    // If the browser does not support the getElements method,
    // leave things as they are, and rely on a fallback style.
    if (!divs)
        return;

    var iCtrlIdx = 0;
    for (var iIdx = 0; iIdx < divs.length; iIdx++)
    {
        if (divs[iIdx].className == 'TabCtrl')
        {
            // Intended tab control found, locate its contents
            iCtrlIdx++;
            var tabspanels = divs[iIdx].getElementsByTagName('DIV');
            var tabs = new Array(0);
            var panels = new Array(0);

            for (var iIdx2 = 0; iIdx2 < tabspanels.length; iIdx2++)
            {
                if (tabspanels[iIdx2].className == 'TabCtrlTab')
                    tabs[tabs.length] = tabspanels[iIdx2];
                else if (tabspanels[iIdx2].className == 'TabCtrlPanel')
                    panels[panels.length] = tabspanels[iIdx2];
            }

            // Construct a replacement HTML content for the tab control,
            // which employs a table to display the contents
            var ctrlText = '<table id="ctrl' + iCtrlIdx + '" class="TabCtrl" cellpadding=0 cellspacing=0>';
            ctrlText += '<tr class="TabRow" onclick="tabAction();">';
            for (iIdx2 = 0; iIdx2 < tabs.length; iIdx2++)
            {
                // Each tab cell is actually a nested table with 2 rows containing
                // 1 cell: the bottom cell is the tab, the top cell functions as
                // a spacer, giving the tab some freedom to extend upwards
                ctrlText += '<td><table class="TabStructure" cellpadding="0" cellspacing="0"><tr><td id="ctrl' + iCtrlIdx + 'dummy' + iIdx2 + '" class="TabSpacer"></td></tr><tr>';
                ctrlText += '<td id="ctrl' + iCtrlIdx + 'tab' + iIdx2 + '" class="Tab">';
                ctrlText += tabs[iIdx2].innerHTML;
                ctrlText += '</td>';
                ctrlText += '</tr></table></td>';
            }

            // Add a transparent inert dummy cell to the right that tries
            // to claim the complete with, forcing the other tabs to shrink
            // to their minimum width (the width of the text)
            ctrlText += '<td width=100% bgcolor=transparent></td>';

            ctrlText += '</tr><tr class="PanelRow">';
            for (iIdx2 = 0; iIdx2 < panels.length; iIdx2++)
            {
                ctrlText += '<td id="ctrl' + iCtrlIdx + 'panel' + iIdx2 + '" class="Panel" colspan=' + (tabs.length + 1) + '>'; // colspan +1 for the dummy!
                ctrlText += panels[iIdx2].innerHTML;
                ctrlText += '</td>';
            }

            // Add an invisible dummy panel associated with the dummy tab cell
            ctrlText += '<td style="display: none;"></td>';

            ctrlText += '</tr></table>';

            // Replace the content of the control
            divs[iIdx].innerHTML = ctrlText;

            // Activate the first tab of each control
            activateTab('ctrl' + iCtrlIdx, 0);
        }
    }
}

/* Ids are formatted as <ctrlId><idTag><idNo>, like 'ctrl2tab3'.
 * This function returns the ctrlId for a specific idTag.
 */
function getCtrlIdFromId(id, idTag)
{
    return id.substring(0, id.indexOf(idTag, 0));
}

/* Ids are formatted as <ctrlId><idTag><idNo>, like 'ctrl2tab3'.
 * This function returns the idNo for a specific idTag.
 */
function getIdNoFromId(id, idTag)
{
    return 1 * id.substring(id.indexOf(idTag, 0) + idTag.length, id.length);
}

function getCtrlIdFromTabId(tabId)      { return getCtrlIdFromId(tabId, 'tab'); }
function getCtrlIdFromPanelId(panelId)  { return getCtrlIdFromId(panelId, 'panel'); }
function getTabNoFromTabId(tabId)       { return getIdNoFromId(tabId, 'tab'); }
function getPanelNoFromPanelId(panelId) { return getIdNoFromId(panelId, 'panel'); }
function getDummyNoFromDummyId(dummyId) { return getIdNoFromId(dummyId, 'dummy'); }

/* This event handler is called when the mouse is clicked in
 * the row of tabs. It then extracts which tab of which control
 * is clicked, and calls activateTab() to select that tab.
 */
function tabAction()
{
    var src = window.event.srcElement;
    if (src.tagName == 'TD' && src.className == 'Tab')
    {
        activateTab(getCtrlIdFromTabId(src.id), getTabNoFromTabId(src.id));
    }
}

/* This function changes the classes of the tabs and panels
 * involved, so that the appropriate styles will be applied,
 * and the intended tab looks selected. 
 */
function activateTab(ctrlId, tabNo)
{
    var ctrl = document.getElementById(ctrlId);
    var tabspanels = ctrl.getElementsByTagName('TD');

    // Go through all tabs in the control, activate the
    // tab/panel selected (if necessary) and deactive the other
    // tabs/panels (if necessary)
    for (var iIdx = 0; iIdx < tabspanels.length; iIdx++)
    {
        if (tabspanels[iIdx].className == 'Tab')
        {
            if (getTabNoFromTabId(tabspanels[iIdx].id) == tabNo)
            {
                tabspanels[iIdx].className = 'ActiveTab';
            }
        }
        else if (tabspanels[iIdx].className == 'ActiveTab')
        {
            if (getTabNoFromTabId(tabspanels[iIdx].id) != tabNo)
            {
                tabspanels[iIdx].className = 'Tab';
            }
        }
        if (tabspanels[iIdx].className == 'TabSpacer')
        {
            if (getDummyNoFromDummyId(tabspanels[iIdx].id) == tabNo)
            {
                tabspanels[iIdx].className = 'ActiveTabSpacer';
            }
        }
        else if (tabspanels[iIdx].className == 'ActiveTabSpacer')
        {
            if (getDummyNoFromDummyId(tabspanels[iIdx].id) != tabNo)
            {
                tabspanels[iIdx].className = 'TabSpacer';
            }
        }
        else if (tabspanels[iIdx].className == 'Panel')
        {
            if (getPanelNoFromPanelId(tabspanels[iIdx].id) == tabNo)
            {
                tabspanels[iIdx].className = 'ActivePanel';
            }
        }
        else if(tabspanels[iIdx].className == 'ActivePanel')
        {
            if (getPanelNoFromPanelId(tabspanels[iIdx].id) != tabNo)
            {
                tabspanels[iIdx].className = 'Panel';
            }
        }
    }
}

